# API Docs
Retorna informações das organizações relacionadas com o produto informado, com suas respectivas permissões de acesso (elementos), servidores de produção e seus mnemônicos.
## Produto requisitado:

| Nome                     | url            | uri                                      |
|--------------------------|----------------|------------------------------------------|
| Organizations Management | om.teknisa.com | https://om.teknisa.com/backend/index.php |

## Verbos HTTPs permitidos:
POST

## Informações de acesso:

O acesso ao OM pode ser feito via integração da Lib (Teknisa Bibliotecas), a partir do produto requisitante. O uso da lib é indicado pois a mesma já tem
as configurações e dados adequados para que a integração seja feita de
maneira correta. Para acessar as informações disponibilizadas na API é necessário realizar duas requisições HTTP.

A primeira requisição consiste em
obter o token de acesso ao produto OM. A segunda, já em posse do token, o
requisitante acessa as rotas disponibilizadas na API. As requisições devem ser
feitas a partir do backend do produto requisitante. Para realizar as requisições HTTP novamente é indicado o uso da Lib. O
arquivo [Utilities.php](http://code.zeedhi.com/teknisa/libbackend/blob/master/src/Teknisa/Libs/Util/Utilities.php) da Lib contém o método **_callWebService_** que é
responsável por realizar as chamadas HTTP utilizando a biblioteca cURL.

> <span style="color: red;">**Nota:**</span> Para os produtos que não fazem uso da Lib, é necessário replicar o metodo **_callWebService_**.


 Método **_callWebService_**:

```php
public function callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected = false)
{
    set_time_limit(180);
    $header = array();
    $header[] = 'OAuth-Token: ' . $token;
    $header[] = 'OAuth-SecretId: ' . $privateKey;
    if($keepConnected) {
        $header[] = 'OAuth-KeepConnected: ' . $keepConnected;
    }
    $header[] = 'Content-Type: application/json';
    $header[] = 'Content-Length: ' . strlen(json_encode($data));

    $ch = curl_init();
    curl_setopt($ch, CURLOPT_HTTPHEADER, $header);
    curl_setopt($ch, CURLOPT_URL, $url . $requestType);
    curl_setopt($ch, CURLOPT_POST, true);
    curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($data));
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 10);
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);

    $result = curl_exec($ch);
    $result = json_decode($result, true);
    $error = curl_error($ch);
    curl_close($ch);

    if (!empty($result['error'])) {
        throw new \Exception($result['error']);
    } else if (!empty($error)) {
        throw new \Exception($error);
    } else {
        return isset($result['dataset'][$dataSetResultPos]) ? $result['dataset'][$dataSetResultPos] : null;
    }
}
```

**Parâmetros**

| parâmetro      | tipo   | descrição                                                                                                                          |
|----------------|--------|------------------------------------------------------------------------------------------------------------------------------------|
|  $token        | String | Token de acesso necessário para acessar rotas autenticadas. Podendo ser null quando a requisição for feita para requisitar o mesmo |
| $requestType   | String | A rota no qual o requisitante está tentando acessar                                                                                |
| $data          | Array  | Dados do corpo da requisição.                                                                                                      |
| $dataResultPos | String | Nome do dataset no qual o requisitante irá acessar os dados na resposta da requisição.                                             |
| $privateKey    | String | Chave de acesso (client secret) necessária para acessar o produto (OM)                                                             |
| $uri           | String | uri para qual vai ser feita a requisição HTTP (do Produto OM no caso)                                                              |




### Procedimento

Sempre será necessário realizar duas requisições HTTP para obter informações da API. Uma requisição para solicitar o token e a segunda para acessar os dados da API.

### Dados

Para acessar o OM é necessário informar os `client id` e `client secret`. Os valores das duas variáveis podem ser encontrada no arquivo [config.json](http://code.zeedhi.com/teknisa/libbackend/blob/master/config/config.json) da Lib.

## Rotas

```
/lib_requestToken
```
Método `POST`

request type  

dataset response `token`

###### Exemplo de uso:

```php
$token = null; 
$requestType = '/lib_requestToken'; 
$data = array(
    'CLIENT_ID' => 'cd7e85178153by60953518945f9hakt19a50a4',
    'CLIENT_SECRET' => 'lib5449996c21f780fe30badd6521931a5sac9'
); 
$dataSetResultPos = 'token'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$uri = 'https://om.teknisa.com/backend/index.php'; 
$keepConnected = false;

$data = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected);
```

### Retorno

```
array(1) {
  [0]=>
  string(38) "32746369a147f4e47ceb2305cf55c3c20a9582"
}
```
<br/>
<hr/>
<br/>

```
/api/findOrganizationInformation
```
Método `POST`

request type `FilterData`

FilterData `PRODUCT_ID` `SHOW_SERVER_INFO (Para não retornar as informações de servidor, o valor deve ser false (boolean)`

dataset response `organizationInformation`

###### Exemplo de uso:

```php
$token = $data[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findOrganizationInformation'; 
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $productId),
    )
); 
$dataSetResultPos = 'organizationInformation'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$uri = 'https://om.teknisa.com/backend/index.php'; 
$keepConnected = false;

$organizationInformation = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected);
```

### Retorno

```
array(39) {
  [0]=>
  array(9) {
    ["NRORG"]=>
    string(4) "1828"
    ["NMORGANIZACAO"]=>
    string(27) "ALENTEJO COZINHA PORTUGUESA"
    ["NMRAZSOCIALORG"]=>
    string(23) "R. L. J. DA COSTA - EPP"
    ["IDORGSAAS"]=>
    string(1) "N"
    ["NMPRODUTO"]=>
    string(23) "Odhen Retail Management"
    ["NRPRODUTO_ORG_ID"]=>
    string(4) "1181"
    ["SERVER_ID"]=>
    string(4) "3556"
    ["SERVER_NAME"]=>
    string(28) "Server Alentejo - Production"
    ["SERVER_PATH"]=>
    string(25) "/var/www/teknisa/alentejo"
  }
  ...
```
<br/>
<hr/>
<br/>

```
/api/findServerMnemonic
```
Método `POST`

request type `FilterData`

FilterData `PRODUCT_ID` `SERVER_ID`

dataset response `serverMnemonic`

###### Exemplo de uso:

```php
$token = $data[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/findServerMnemonic'; 
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $productId),
        array('name' => 'SERVER_ID', 'operator' => '=', 'value' => $serverId),
    )
); 
$dataSetResultPos = 'serverMnemonic'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$uri = 'https://om.teknisa.com/backend/index.php'; 
$keepConnected = false;

$serverMnemonic = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected);
```

### Retorno

```
array(33) {
  [0]=>
  array(6) {
    ["SERVER_ID"]=>
    string(4) "3556"
    ["MNEMONIC"]=>
    string(8) "BIRT_URL"
    ["LABEL"]=>
    string(28) "URL WHERE BIRT IS CONFIGURED"
    ["MNEMONIC_ID"]=>
    string(3) "228"
    ["VALUE"]=>
    string(36) "https://manage.alentejo.com.br:8080/"
    ["IS_DEFAULT_VALUE"]=>
    string(1) "M"
  }
  ...

  ```
<br/>
<hr/>
<br/>

```
/api/findProductElement
```
Método `POST`

request type `FilterData`

FilterData `PRODUCT_ID` `NRPRODUTO_ORG_ID`

dataset response `productElement`

###### Exemplo de uso:

```php
$token = $data[0];  // token obtido através da rota 'lib_requestToken'
$requestType = '/api/productElement'; 
$data = array(
    'requestType' => 'FilterData',
    'filter' => array(
        array('name' => 'PRODUCT_ID', 'operator' => '=', 'value' => $productId),
        array('name' => 'NRPRODUTO_ORG_ID', 'operator' => '=', 'value' => $nrProductOrgId),
    )
); 
$dataSetResultPos = 'productElement'; 
$privateKey = 'lib5449996c21f780fe30badd6521931a5sac9'; 
$uri = 'https://om.teknisa.com/backend/index.php'; 
$keepConnected = false;

$productElement = callWebService($token, $requestType, $data, $dataSetResultPos, $privateKey, $url, $keepConnected);
```

### Retorno

```
array(227) {
  [0]=>
  array(8) {
    ["ID"]=>
    string(4) "3667"
    ["LABEL"]=>
    string(19) "Integração AudTax"
    ["UNIQUE_ID"]=>
    string(22) "9088430364064306921308"
    ["NRORDEM"]=>
    string(1) "0"
    ["NRPRODUTO_ID"]=>
    string(3) "305"
    ["PARENT_ID"]=>
    string(4) "2490"
    ["PARENT_LABEL"]=>
    string(25) "Importação/Exportação"
    ["DTVALIDADE"]=>
    NULL
  }
  ...

